﻿Imports System.Reflection
Imports System.Windows.Controls
Imports System.Windows.Navigation

Imports System.ComponentModel.Composition
Imports System.ComponentModel.Composition.Hosting

Imports Utils.Silverlight.MVVM.Navigation

Namespace Utils.Silverlight.MVVM
    <Export()>
 Public Class CompositionNavigationContentLoader
        Implements INavigationContentLoader
        ''' <summary>
        ''' A list of factories for all available Views.
        ''' </summary>
        <ImportMany(GetType(IView))>
        Public Property ViewExports() As IEnumerable(Of ExportFactory(Of IView, IViewMetadata))

        ''' <summary>
        ''' A list of factories for all available ViewModels.
        ''' </summary>
        <ImportMany(GetType(IViewModel))>
        Public Property ViewModelExports() As IEnumerable(Of ExportFactory(Of IViewModel, IViewModelMetadata))

        ''' <summary>
        ''' Load is enabled by default.
        ''' </summary>
        ''' <param name="targetUri"></param>
        ''' <param name="currentUri"></param>
        ''' <returns></returns>
        Public Function CanLoad(ByVal targetUri As Uri, ByVal currentUri As Uri) As Boolean Implements INavigationContentLoader.CanLoad
            Return True
        End Function

        ''' <summary>
        ''' Don't do anything when canceling.
        ''' </summary>
        ''' <param name="asyncResult"></param>
        Public Sub CancelLoad(ByVal asyncResult As IAsyncResult) Implements INavigationContentLoader.CancelLoad
            Return
        End Sub

        ''' <summary>
        ''' Begins asynchronous loading of the content for the specified target URI.
        ''' </summary>
        ''' <param name="targetUri">The URI to load content for.</param>
        ''' <param name="currentUri">The URI that is currently loaded.</param>
        ''' <param name="userCallback">The method to call when the content finishes loading.</param>
        ''' <param name="asyncState">An object for storing custom state information.</param>
        ''' <returns>
        ''' An object that stores information about the asynchronous operation.
        ''' </returns>
        Public Function BeginLoad(ByVal targetUri As Uri, ByVal currentUri As Uri, ByVal userCallback As AsyncCallback, ByVal asyncState As Object) As IAsyncResult Implements INavigationContentLoader.BeginLoad
            ' Convert to a dummy relative Uri so we can access the host.
            Dim relativeUri = New Uri("http://" & targetUri.OriginalString, UriKind.Absolute)

            ' Get the factory for the ViewModel.
            Dim viewModelMapping = ViewModelExports.FirstOrDefault(
                Function(o) o.Metadata.Key.Equals(relativeUri.Host, StringComparison.OrdinalIgnoreCase))
            If viewModelMapping Is Nothing Then
                Throw New InvalidOperationException(String.Format("Unable to navigate to: {0}. Could not locate the ViewModel.", targetUri.OriginalString))
            End If

            ' Get the factory for the View.
            Dim viewMapping = ViewExports.FirstOrDefault(
                Function(o) o.Metadata.ViewModelContract.Equals(viewModelMapping.Metadata.ViewModelContract))
            If viewMapping Is Nothing Then
                Throw New InvalidOperationException(String.Format("Unable to navigate to: {0}. Could not locate the View.", targetUri.OriginalString))
            End If

            ' Resolve both the View and the ViewModel.
            Dim viewFactory = viewMapping.CreateExport()
            Dim view = TryCast(viewFactory.Value, Control)
            Dim viewModelFactory = viewModelMapping.CreateExport()
            Dim viewModel = TryCast(viewModelFactory.Value, IViewModel)

            ' Attach ViewModel to View.
            view.DataContext = viewModel
            viewModel.OnLoaded()

            ' Get navigation values.
            Dim values = viewModelMapping.Metadata.GetArgumentValues(targetUri)
            viewModel.OnNavigated(values)

            If TypeOf view Is Page Then
                Dim page = TryCast(view, Page)
                page.Title = viewModel.GetTitle()
            ElseIf TypeOf view Is ChildWindow Then
                Dim window = TryCast(view, ChildWindow)
                window.Title = viewModel.GetTitle()
            End If

            ' Do not navigate if it's a ChildWindow.
            If TypeOf view Is ChildWindow Then
                ProcessChildWindow(TryCast(view, ChildWindow), viewModel)
                Return Nothing
            Else
                ' Navigate because it's a Control.
                Dim result = New CompositionNavigationAsyncResult(asyncState, view)
                userCallback(result)
                Return result
            End If
        End Function

        ''' <summary>
        ''' Show the ChildWindow and attach the CloseView event.
        ''' </summary>
        ''' <param name="window"></param>
        ''' <param name="viewModel"></param>
        Private Sub ProcessChildWindow(ByVal window As ChildWindow, ByVal viewModel As IViewModel)
            ' Close the ChildWindow if the ViewModel requests it.
            Dim closableViewModel = TryCast(viewModel, IClosableViewModel)
            If closableViewModel IsNot Nothing Then
                AddHandler closableViewModel.CloseView, Sub(s, e) window.Close()
            End If

            ' Show the window.
            window.Show()
        End Sub


        ''' <summary>
        ''' Finished loading, proceed.
        ''' </summary>
        ''' <param name="asyncResult"></param>
        ''' <returns></returns>
        Public Function EndLoad(ByVal asyncResult As IAsyncResult) As LoadResult Implements INavigationContentLoader.EndLoad
            Return New LoadResult((TryCast(asyncResult, CompositionNavigationAsyncResult)).Result)
        End Function
    End Class
End Namespace
